// box.cpp -- handles box-specific devices

#include "box.h"

#ifdef WIN32
#else

#ifdef __DVB_API_3__
#include "/home/billy/e2ssl/mipsel-unknown-linux-gnu/mipsel-unknown-linux-gnu/sys-root/usr/include/linux/dvb/dmx.h"
#else
#include "/home/billy/btkcam/dmx.h"  //for older powerpc machines (DM500, DM7000, DM7020)
#endif

#include <unistd.h>
#include <sys/ioctl.h>

#endif

#include <string.h>
#include <fcntl.h>
#include "box.h"
#include "common.h"
#include "cam.h"
#include "ca.h"


struct box_devices box::devices[BOX_COUNT] = {
#ifdef __DVB_API_3__
	/* dreambox (dvb-api-3)*/	{ "/dev/dvb/adapter0/ca%d", "/dev/dvb/adapter0/demux%d", "/var/tmp/camd.socket" },
#else	
	/* dreambox (dvb-api-1)*/	{ "/dev/dvb/card0/ca%d", "/dev/dvb/card0/demux%d", "/var/tmp/camd.socket" },
#endif
	/* triple dragon */		{ "/dev/stb/tddemux%d", "/dev/stb/tddemux%d", "/tmp/camd.socket" },
	/* dgstation */			{ "/dev/demuxapi", "/dev/demuxapi", "/tmp/camd.socket" }, 
	/* dbox2 (dvb-api-1)*/		{ "/dev/dvb/card0/ca%d", "/dev/dvb/card0/demux%d", "/var/tmp/camd.socket" }
};

int box::ca_device_fd[8] = {0, 0, 0, 0, 0, 0, 0, 0};
unsigned int box::type = 0; //default to dreambox
	
/* static */ char *box::get_ca_device_path( void )
{
	return devices[type].ca_device_path;
}

/* static */ char *box::get_demux_device_path( void )
{
	return devices[type].demux_device_path;
}

/* static */ char *box::get_cam_socket_path( void )
{
	return devices[type].cam_socket_path;
}

/* static */ void box::close_ca_devices( void )
{
	for(int i=0;i<8;i++)
	{
		if( ca_device_fd[i] )
		{
			close( ca_device_fd[i] );
			ca_device_fd[i] = 0;
		}
	}	
}

/* static */ int box::get_ca_device( int device_index, service* handler )
{
	int ca_fd;
	char device_path[256];
	int ca_offset=0;

	if (type==3) ca_offset=1;		//dbox2 icin lazim

	if( ca_device_fd[device_index+ca_offset] ) 
		return ca_device_fd[device_index+ca_offset];
	
	sprintf(device_path, devices[type].ca_device_path, device_index+ca_offset);
	if((ca_fd = open(device_path, O_RDWR)) <= 0)
	{
		_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:-]: Error opening CA device %d.", device_index+ca_offset), 1);
		return 0;
	}
	
	//box dependent initialization
	switch( type )
	{
		case 0: //dreambox
		case 3: //dbox2
			break;

		case 1: //triple-dragon
			ca_descr_t descr;
			descr.index  = 0;
			descr.parity = 2; /* init vector */
			memset(descr.cw, 0, 8);
			
			ioctl(ca_fd, TD_CA_SET_DESCR, &descr);
			ioctl(ca_fd, TD_CA_INTERN_ENABLE);
			break;
		
		case 2: //dgstation
			ca_descr_t dgstation_descr;
			dgstation_descr.index  = 0;
			dgstation_descr.parity = 1; /* init vector */
			memset(dgstation_descr.cw, 0, 8);
			
			ioctl(ca_fd, DGSTATION_CA_SET_DESCR, &dgstation_descr);
			break;
	}
	
	ca_device_fd[device_index+ca_offset] = ca_fd;
	return ca_fd;
}

/* static */ int box::get_demux_device( int device_index, service* handler )
{
	int demux_fd;
	char device_path[256];
	
	sprintf(device_path, devices[type].demux_device_path, device_index);
	if((demux_fd = open(device_path, O_RDWR)) <= 0)
	{
		_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Error opening demux device %d.", handler->descramble_index, device_index), 1);
		return 0;
	}
	return demux_fd;
}

/* static */ int box::close_demux_device( service* handler )
{
	if (handler->ecm_demux_fd>0) close(handler->ecm_demux_fd);
	handler->ecm_demux_fd=-1;
}
	
/* static */ int box::set_ca_pid( int device_index, ca_pid_t* service_pid, service* handler )
{
	int set_status;
	switch( type )
	{
		case 0: //dreambox
		case 3: //dbox2
			set_status = ioctl(get_ca_device(device_index, handler), CA_SET_PID, service_pid);
			break;
		
		case 1: //triple-dragon
			set_status = ioctl(get_ca_device(device_index, handler), TD_CA_SET_PID, service_pid);
			break;
			
		case 2: //dgstation
			set_status = ioctl(get_ca_device(device_index, handler), DGSTATION_CA_SET_PID, service_pid);
			break;
	}
	return set_status;
}

/* static */ int box::start_filter( int demux_descriptor, unsigned short pid, unsigned char table, unsigned char mask, unsigned int timeout, service* handler)
{
	switch( type )
	{
		case 0: //dreambox
		case 3: //dbox2
#ifdef __DVB_API_3__	//DM7025, DM800HD, DM8000HD
			struct dmx_sct_filter_params demux_parameters;
#else					//DM500, DM600, DM7000, DM7020
			struct dmxSctFilterParams demux_parameters;
#endif
			stop_filter(demux_descriptor, handler); //in case it's running
			memset(&demux_parameters, 0, sizeof(demux_parameters)); //clear structure
			demux_parameters.pid = pid;
			demux_parameters.filter.filter[0] = table;
			demux_parameters.filter.mask[0] = mask;
			demux_parameters.timeout = timeout;
			demux_parameters.flags = DMX_IMMEDIATE_START;
			if( ioctl(demux_descriptor, DMX_SET_FILTER, &demux_parameters) < 0 )
			{
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Could not set demux filter.", handler->descramble_index), 1);
				return -1;
			}
			break;
			
		case 1: //triple-dragon
			struct demux_filter_para td_demux_parameters;
			stop_filter(demux_descriptor, handler); //in case it's running
			memset(&td_demux_parameters, 0, sizeof(td_demux_parameters)); //clear structure
			td_demux_parameters.pid = pid;
			td_demux_parameters.filter[0] = table;
			td_demux_parameters.mask[0] = mask;
			td_demux_parameters.timeout = timeout;
			td_demux_parameters.positive[0]  = 0x00;
			td_demux_parameters.filter_length = 4;
			if( ioctl(demux_descriptor, TD_DEMUX_FILTER_SET, &td_demux_parameters) < 0 )
			{
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Could not set demux filter.", handler->descramble_index), 1);
				return -1;
			}
			if( ioctl(demux_descriptor, TD_DEMUX_START) < 0 )
			{
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Could not start demux filter.", handler->descramble_index), 1);
				return -1;
			}
			break;
			
		case 2: //dgstation
			struct demux_filter_para_dgstation dgstation_demux_parameters;
			stop_filter(demux_descriptor, handler); //in case it's running
			memset(&dgstation_demux_parameters, 0, sizeof(dgstation_demux_parameters)); //clear structure
			dgstation_demux_parameters.pid = pid;
			dgstation_demux_parameters.filter[0] = table;
			dgstation_demux_parameters.mask[0] = mask;
			dgstation_demux_parameters.timeout = timeout;
			dgstation_demux_parameters.filter_length = 4;
			if( ioctl(demux_descriptor, DGSTATION_DEMUX_FILTER_SET, &dgstation_demux_parameters) < 0 )
			{
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Could not set demux filter.", handler->descramble_index), 1);
				return -1;
			}

			if( ioctl(demux_descriptor, DGSTATION_DEMUX_START) < 0 )
			{
				_slog(handler->log_trans, _DEBUG_BASIC, 1, _logmf("cam[di:%d]: Could not start demux filter.", handler->descramble_index), 1);
				return -1;
			}
			break;
	}

	return 0;
}

/* static */ void box::stop_filter( int demux_descriptor, service* handler )
{
	int dmx_command;
	switch( type )
	{
		case 0: //dreambox
		case 3: //dbox2
			dmx_command = DMX_STOP;
			break;
		case 1: //triple-dragon
			dmx_command = TD_DEMUX_STOP;
			break;
		case 2: //dgstation
			dmx_command = DGSTATION_DEMUX_STOP;
			break;
	}
	ioctl(demux_descriptor, dmx_command);
}

/* static */ void box::write_control_words( service* handler )
{
	ca_descr_t ca_descr;
	int i;
	unsigned char nullx[8];
	int dmx_command;

	//if ( !fexists("/tmp/btk.info") ) return;

	switch( type )
	{
		case 0: //dreambox
		case 3: //dbox2
			dmx_command = CA_SET_DESCR;
			break;
		case 1: //triple-dragon
			dmx_command = TD_CA_SET_DESCR;
			break;
		case 2: //dgstation
			dmx_command = DGSTATION_CA_SET_DESCR;
			break;
	}
	
	_slog(handler->log_trans, _DEBUG_BASIC, 2, "Control words:", 1);
	handler->cw_0[3]=handler->cw_0[0]+handler->cw_0[1]+handler->cw_0[2];
	handler->cw_1[3]=handler->cw_1[0]+handler->cw_1[1]+handler->cw_1[2];
	handler->cw_0[7]=handler->cw_0[4]+handler->cw_0[5]+handler->cw_0[6];
	handler->cw_1[7]=handler->cw_1[4]+handler->cw_1[5]+handler->cw_1[6];
	memset(nullx,0,8);
	

	if( memcmp(handler->cw_0, handler->last_cw_0, 8) && memcmp(nullx,handler->cw_0,8) )
	{
		ca_descr.index = handler->descramble_index;
		ca_descr.parity = 0;
		memcpy(ca_descr.cw, handler->cw_0, 8);
		for(i=0;i<8;i++)
		{
			if( handler->descramble_mask & (1 << i) )
			{
				if( ioctl(get_ca_device(i, handler), dmx_command, &ca_descr) < 0 )
				{
					_slog(handler->log_trans, _DEBUG_BASIC, 2, _logmf("cam[di:%d] Could not set control word 0.", handler->descramble_index), 1);
					return;
				}
			}
		}
		_flog(handler->log_trans, _DEBUG_BASIC, 3, "CW0 (    SET): ", 0, handler->cw_0, 8, 0, 0, 1);
	}
	else _flog(handler->log_trans, _DEBUG_BASIC, 3, "CW0 (NOT SET): ", 0, handler->cw_0, 8, 0, 0, 1);
	
	if( memcmp(handler->cw_1, handler->last_cw_1, 8) && memcmp(nullx,handler->cw_1,8) )
	{
		ca_descr.index = handler->descramble_index;
		ca_descr.parity = 1;
		memcpy(ca_descr.cw, handler->cw_1, 8);
		
		for(i=0;i<8;i++)
		{
			if( handler->descramble_mask & (1 << i) )
			{
				if( ioctl(get_ca_device(i, handler), dmx_command, &ca_descr) < 0 )
				{
					_slog(handler->log_trans, _DEBUG_BASIC, 2, _logmf("cam[di:%d] Could not set control word 1.", handler->descramble_index), 1);
					return;
				}
			}
		}
		_flog(handler->log_trans, _DEBUG_BASIC, 3, "CW1 (    SET): ", 0, handler->cw_1, 8, 0, 0, 2);
	}
	else _flog(handler->log_trans, _DEBUG_BASIC, 3, "CW1 (NOT SET): ", 0, handler->cw_1, 8, 0, 0, 2);
}

